// Copyright by HandHeld Speech LLC, 2007

#ifndef hhsClnt_h
#define hhsClnt_h

#ifdef WIN32

#ifdef _USRDLL					// build hhs.dll and hhs.lib
#define WIN32_LEAN_AND_MEAN		// Exclude rarely-used stuff from Windows headers
#define HHSCLNT_API		__declspec(dllexport)

#else
#define HHSCLNT_API		__declspec(dllimport)
#endif

#else

#define HHSCLNT_API extern
#define INVALID_HANDLE_VALUE  0
#define HANDLE LongInt

#ifndef Sleep
#define Sleep(x) usleep( (x)<<10 )
#endif
#endif

// this is a C interface -- name mangling is OFF
#ifdef __cplusplus

class User;
class Utterance;
class RuleData;
class WordData;
class WordDictionary;
class MoodModels;
class ClientInterimResult;
class MotionStream;

// is() macro for C++ default arugments
#define is(x)	=x

extern "C" {

#else

// truly only C
struct User;
struct Utterance;
struct RuleData;
struct WordData;
struct WordDictionary;
struct ClientInterimResult;
struct MoodModels;
struct MotionStream;

typedef struct User User;
typedef struct Utterance Utterance;
typedef struct RuleData RuleData;
typedef struct WordData WordData;
typedef struct WordDictionary WordDictionary;
typedef struct ClientInterimResult ClientInterimResult;
typedef struct MoodModels MoodModels;
typedef struct MotionStream MotionStream;

typedef struct User* PUser;
typedef struct Utterance* PUtterance;
typedef struct RuleData* PRuleData;
typedef struct WordData* PWordData;
typedef struct WordDictionary* PWordDictionary;
typedef struct ClientInterimResult* PClientInterimResult;
typedef struct MoodModels* PMoodModels;
typedef struct MotionStream* PMotionStream;

// no default arguments in C
#define is(x)

#endif

typedef void (*OnSpeechCallback)( Utterance* utt );


//  There are 4 mood types currently monitored:
#define CALM_MOOD	0
#define ANGRY_MOOD	1
#define SAD_MOOD	2
#define HAPPY_MOOD	3
#define MAX_MOOD	4
 
// listen Flag bits
#define LstnLISTENING		0x0001
#define LstnCOMPLETE		0x0002
#define LstnABORT			0x0004
#define LstnMODAL			0x0040
#define LstnCONTINUED		0x0080
#define LstnCALLBACK		0x0100
#define LstnKEEPALTPRONS    0x0200
#define LstnKEEPWAVDATA		0x1000
#define LstnKEEPPWRDATA		0x2000
#define LstnKEEPEMODATA	    0x4000

// LanguageID's
#define ENGLISH_LANGID		1
#define SPANISH_LANGID		2
#define	GERMAN_LANGID		3
#define PORTUGUESE_LANGID	4
#define	JAPANESE_LANGID		5
//#define	RUSSIAN_LANGID		6
#define	MAX_LANGID			6

// SemanticId types
#define RE_TERMINAL			0
#define RE_WORD				1
#define RE_WORD_MORE_IDS	2
#define RE_ENTER_RULE		3
#define RE_RETURN_RULE		4
#define UNUSED_SEMANTIC_ID		0x80000000

// 32bit pointer world: ResultElement's are 3 unsigned's long
// 64bit pointer world: ResultElement's are 4 unsigned's long

#ifndef REsize
// you could cast ResultElement unsigned ints to this struct:
struct ResultElement
{
	unsigned type;
	unsigned semanticId;
	union {
		WordData* word;
		RuleData* rule;
	};
};
#define REsize ( sizeof(ResultElement) / sizeof(unsigned) )
#endif

// modes for hhsWriteWavFile()
#define WAVE_MODE_11M16		0x07	// nist .wav file format (11025hz mono 16bit)
#define WAVE_MODE_R11M16	0x47		// riff is 64+nist value (11025hz mono 16bit)
#define WAVE_MODE_APPEND	0x100	// Can '|' with other modes

// hhsPlay...() envelopes:
// ( 0 - dropping statement, 1 - neutral, 2 - rising question )
// all can be or'ed with BARGE_IN and PLAY_SYNC
#define BARGE_IN		0x0100
#define PLAY_SYNC		0x0200
#define PITCH_DROPPING	0x0000
#define PITCH_STEADY	0x0001
#define PITCH_RISING	0x0002

// Style bits are more translating strings to prons:
// (except for case 0x20, WholeWords replaced with spelling based prons)
#define BEST_RECOGNIZED_PRONS			0x00
#define PREFER_TTS_LIB_PRONS			0x10
#define BEST_RECOGNIZED_PRONS_W			0x20 (WholeWords replaced with a fake "W" phoneme)
#define BEST_RECOGNIZED_PRONS_DOTLESS	0x30 (omit '.' (silence) (used pron in guessing))

// listening status and play status return values
#define WAV_DEV_OFF     	    0
#define WAV_DEV_PAUSED	 	    2
#define WAV_DEV_PAUSING			3
#define WAV_DEV_RESTART        	4		// only used in waveiop.cpp
#define WAV_DEV_STARTING		5
#define WAV_DEV_ON          	6
#define WAV_DEV_INSPEECH	0x800		// only listen status

// hhsListen return status is either 0 or NODATA_ABORT
#define NODATA_ABORT	-1

HHSCLNT_API int hhsNewGlobalRecognizer( const char *ws is(0) );
// ws = fullpath including hhsDemon.exe

HHSCLNT_API int hhsDeleteGlobalRecognizer();


HHSCLNT_API int hhsHeartBeat();
// a counter that is incremented each demon awakening
// if the demon is healthy, return value should change

HHSCLNT_API void hhsKillDemon();
// very dangerous/rude -- pulls the plug on the demon...
// if aborted all subsequent client API calls fail;
// clients should call hhsDeleteGlobalRecognizer() to cleanup,
// then restart with an hhsNewGlobalRecognizer() followed by
// loading their .voc / .usr / stateMachines...

HHSCLNT_API int hhsLoadWords( const char* pathname );
// ws = fullpath including words.voc

HHSCLNT_API int hhsSaveWords( const char* fname, int filter is(0) );

HHSCLNT_API int hhsNewUserWord( User* usr, Utterance* utt, const char *spell, int penalty is(0), int threshold is(500) );

HHSCLNT_API int hhsLoadUserWordsIntoWordDictionary( User* usr );

HHSCLNT_API int hhsLoadTtsData( const char* fname, int isWords is(0) );
// ws = fullpath including words.tts

HHSCLNT_API int hhsSaveTtsData( const char* fname );

HHSCLNT_API int hhsDeleteTtsData();

HHSCLNT_API int hhsIsValidUser( User* usr );
// checks that the User* is valid() -> use in assert()

HHSCLNT_API int hhsIsValidUtt( Utterance* utt );
// checks that the Utterance* is valid() -> use in assert()

HHSCLNT_API int hhsIsValidRule( RuleData* rl );
// checks that the RuleData* is valid() -> use in assert()

HHSCLNT_API int hhsIsValidWord( WordData* wrd );
// checks that the WordData* is valid() -> use in assert()

HHSCLNT_API int hhsIsActiveWord( WordData* wrd );

HHSCLNT_API int hhsIsValidWordDictionary( WordDictionary* wdict );
// checks that the WordDictionary* is valid() -> use in assert()

HHSCLNT_API int hhsIsValidMoodModel( MoodModels* mm );
// checks that the MoodModels are valid() -> use in assert()

HHSCLNT_API int hhsUttCounter( Utterance* utt );
// uttCounter is strictly increasing, first utt is 1

HHSCLNT_API int hhsStringToPron( char *targ, const char *src, int targlen, short spellMethod, WordDictionary* wd is(0) );
// spellMethod is a flag determining pron production
// legal values are 0, 1, 2, and 5
// 0 causes no spaces, "this is" -> "Disiz"
// 1 casuse ' ', "this is" -> "Dis iz"
// 2 causes '.', "this is" -> "Dis.iz"
// 5 causes "this is" -> "tE AC oE es oE es"


////////////////////////////////////////////////////////////////
// TTS Flag bits
// currently a pitch ENVELOPE:
//		0-> pitch dropping (statement)
//		1-> pitch neutral
//		2-> pitch rising (question)
//
//  next 2 bits:
//		0  ->	lowest pitch
//		4  ->	medium pitch
//		8  ->	high pitch
//		12 ->	mickey mouse
//
// tts bits are confined to 0x00f

////////////////////////////////////////////////////////////////
// Sound production mechanics (bargeIn bits):
//		0x100 -> BargeIn
//		0x200 -> PLAY_SYNC
// production bits are confined to 0xf00

////////////////////////////////////////////////////////////////
// PRON Flag bits
// a result to Pron STYLE:
//		0x00 -> exactPron -- WholeWords replaced with alt pron or pron guesses,
//		0x10 -> case 0 but preferring prons occurring in TTS library
//		0x20 -> exactPron -- WholeWords are replaced with a fake "W" phoneme
//		0x30 -> save as 0 but omit '.' (silence phones) (used pron in guessing)
// so STYLE bits are confined to 0x0f0


////////////////////////////////////////////////////////////////
// esbFlag is the combined ENVELOPE_STYLE_BARGEIN flag:
//		TTS flag | PRON Flag | BarginSync
// confined to 0xfff bits


HHSCLNT_API int hhsPlayPron( const char *src, short esbFlag is(0) );

HHSCLNT_API int hhsPlayUtt( Utterance* utt, short bargein is(0) );

HHSCLNT_API int hhsPlayIthResult( Utterance* utt, int i, short esbFlag is(0) );

HHSCLNT_API int hhsPlayString( const char* ws, short esbFlag is(0), WordDictionary* wdict is(0) );
// sequentially calls stringToPron() then playPron()

HHSCLNT_API int hhsPlayWavFile( const char* fname, unsigned offset is(0), short esb is(0) );

HHSCLNT_API int hhsPlayAbort();

HHSCLNT_API int hhsPlayStatus();

HHSCLNT_API User* hhsNewUser();

HHSCLNT_API int hhsDeleteUser( User* usr );

HHSCLNT_API int hhsLoadUser( User* usr, const char* fname );

HHSCLNT_API int hhsSaveUser( User* usr, const char* fname );

HHSCLNT_API int hhsSingleModels( User* usr );

HHSCLNT_API int hhsDoubleModels( User* usr );

HHSCLNT_API User* hhsGetDefaultUser( short langId );

HHSCLNT_API User* hhsSetDefaultUser( short langId, const char* fname );

HHSCLNT_API int hhsGetUserFilename( User* usr, char *buf, int buflen, short flag is(0) );

HHSCLNT_API RuleData* hhsNewRule( const char* rulname, int penalty is(0), unsigned semanticId is(UNUSED_SEMANTIC_ID) );

HHSCLNT_API int hhsGetRulePenaltyId( RuleData* rule, int* penalty, unsigned* semanticId );

HHSCLNT_API int hhsSetRulePenalty( RuleData* rule, int penalty );

HHSCLNT_API int hhsSetRuleSemanticId( RuleData* rule, unsigned semanticId );

HHSCLNT_API int hhsLoadRule( RuleData* rl, const char* rulbody );

HHSCLNT_API int hhsLoadRules( const char* fname );

HHSCLNT_API int hhsSaveRules( const char* fname );

HHSCLNT_API int hhsDeleteRule( RuleData* rl );

HHSCLNT_API int hhsCleanUpName( char* targ, const char* src, int targlen, short langId );

HHSCLNT_API int hhsGetAbsolutePath( char* targ, const char* srcPath, int targlen );

HHSCLNT_API int hhsSetLogDir( const char *dirPath );

HHSCLNT_API void hhsLog( const char *s1, const char *s2 is(0) );

HHSCLNT_API int hhsSetLogWarnings( short val );

HHSCLNT_API int hhsGetVersion( char* vbuf, int buflen );

HHSCLNT_API int hhsSetLanguageId( short langId );

HHSCLNT_API int hhsSetUseThroatMic( short onOff );

HHSCLNT_API int hhsIsUsingThroatMic();

HHSCLNT_API int hhsSetWaveInput( const char* fileName );

HHSCLNT_API int hhsSetWaveOutput( const char* fileName );

HHSCLNT_API int hhsConnectHalfDuplexCurrentIO();

HHSCLNT_API int hhsConnectFullDuplexCurrentIO();

HHSCLNT_API int hhsWriteWavFileUtt( Utterance* utt, const char* fname, int mode is(WAVE_MODE_R11M16), int nFrmToTrim is(0) );

HHSCLNT_API int hhsWriteLnPwrFile( Utterance* utt, const char* fname );

HHSCLNT_API int hhsListenRollMute( short nBitsToRollRight );

HHSCLNT_API int hhsSetGainWindow( short high );

HHSCLNT_API int hhsSetBestInputGain( Utterance* utt );

HHSCLNT_API int hhsPlayRollMute( short nBitsToRollRight );

HHSCLNT_API int hhsResultCount( Utterance* utt );

HHSCLNT_API int hhsResultIsEarlyOut( Utterance* utt );

HHSCLNT_API int hhsResultsToAscii( Utterance* utt, int startI, char *buf, int buflen );

HHSCLNT_API int hhsResultsToProns( Utterance* utt, int startI, char *buf, int buflen, int style is(1) );

HHSCLNT_API int hhsResultsAreMine( Utterance* utt );

HHSCLNT_API int hhsResultsToUsers( Utterance* utt, User** buf, int buflen );

HHSCLNT_API int hhsResultsToScores( Utterance* utt, int* buf, int buflen );

HHSCLNT_API int hhsResultsToFirstSemanticIds( Utterance* utt, unsigned* buf, int buflen );

HHSCLNT_API int hhsResultsToFirstRules( Utterance* utt, RuleData** buf, int buflen );

HHSCLNT_API int hhsIthResultIsMine( Utterance* utt, int i );

HHSCLNT_API int hhsIthResultToAscii( Utterance* utt, int i, char *buf, int buflen );

HHSCLNT_API int hhsIthResultToPron( Utterance* utt, int i, char *buf, int buflen, int style is(1) );

HHSCLNT_API User* hhsIthResultUser( Utterance* utt, int i );

HHSCLNT_API int hhsIthResultScore( Utterance* utt, int i );

HHSCLNT_API int hhsNumFramesInUtt( Utterance* utt );


HHSCLNT_API int hhsSetKeyValueString( Utterance* utt, const char* key, const char* val );
HHSCLNT_API int hhsSetKeyValueInt( Utterance* utt, const char* key, int val );
HHSCLNT_API int hhsGetKeyValueString( Utterance* utt, const char* key, char* buf, int blen );
HHSCLNT_API int hhsGetKeyValueInt( Utterance* utt, const char* key );
HHSCLNT_API int hhsGetKeyValueType( Utterance* utt, const char* key );
HHSCLNT_API int hhsDeleteKeyValue( Utterance* utt, const char* key );

#define hhsSetPrompt( utt, val ) hhsSetKeyValueString( utt,"prompt", val )
#define hhsGetPrompt( utt, buf, blen ) hhsGetKeyValueString( utt, "prompt", buf, blen )

HHSCLNT_API int hhsDeleteUtt( Utterance* utt );

HHSCLNT_API int hhsRejectUtt( Utterance* utt);

HHSCLNT_API int hhsGetPitch( User* usr );

HHSCLNT_API int hhsSetPitch( User* u, int p );


// new Mood/Emo API's
HHSCLNT_API MoodModels* hhsNewMoodModels();
HHSCLNT_API int hhsDeleteMoodModels( MoodModels* );
HHSCLNT_API int hhsLoadMoodModels( MoodModels*, const char* fname );
HHSCLNT_API int hhsSaveMoodModels( MoodModels*, const char* fname );
HHSCLNT_API int hhsGetNumMoods( MoodModels* mm );
HHSCLNT_API int hhsGetMoodName( MoodModels* mm, int i, char *buf, int buflen );
HHSCLNT_API int hhsScoreMood( Utterance* utt, MoodModels *mm, int *arrMoodScr, int arrlen );
HHSCLNT_API int hhsGetEmoLambdaInfo( Utterance* u, unsigned short* arrayOf2Shorts, int lenArray );

HHSCLNT_API int hhsSetSignalProcessing( User* usr, int updateRoll is(0) );

HHSCLNT_API int hhsGetUserData( User* usr, int *data16Ints, int datalen );

HHSCLNT_API int hhsSetUserData( User* usr, int *data16Int, int datalen );

HHSCLNT_API int hhsSetMaxResultCount( short nCnt );

HHSCLNT_API int hhsScaleUttDetectionValues( short percent );

HHSCLNT_API int hhsGetUttDetectionValues( unsigned short* arrayOfShorts, int arraylen );

HHSCLNT_API int hhsSetUttDetectionValues( int low, int high );

HHSCLNT_API int hhsSetMaxFramesInUtt( short nframes );

HHSCLNT_API int hhsSetNumFramesOfTerminalSilence( short nframes );

HHSCLNT_API int hhsSetNoiseInjectionThresh( short value );

HHSCLNT_API int hhsWaveInTransientDiscardLength( short newLen );

HHSCLNT_API RuleData* hhsGetRule( const char* name );

HHSCLNT_API int hhsAdaptAccumulate( User* usr, Utterance* utt, const char* prompt, short adaptFlg is(0) );

HHSCLNT_API int hhsAdaptFinish( User* usr, short adaptFlg is(0) );

HHSCLNT_API int hhsAdapt( User* usr, Utterance* utt, const char* buf, short adaptFlg );

HHSCLNT_API int hhsAdaptUsingIthResult( Utterance* utt, int i, short adaptFlg );

HHSCLNT_API int hhsAdaptHappened( User* usr );

HHSCLNT_API int hhsClearAdaptCounts( User* usr );

HHSCLNT_API int hhsBatchAdapt( User* usr, Utterance** utts, short numUtts, short adaptFlg is(0) );

HHSCLNT_API int hhsSetPruneThresh( short t );

HHSCLNT_API int hhsSetMaxAllocDuringRecognition( int mxMem );

HHSCLNT_API User* hhsPickUserAndPitch( Utterance* utt, User* usr1, User* usr2 is(0) );

HHSCLNT_API User* hhsPickUser( Utterance* utt, User** users, int numUsers, int pitch );

HHSCLNT_API	int hhsUpdateUserCousins( User* usr );

HHSCLNT_API int hhsIthResultScoreDetails( Utterance* utt, int i, short *buf, int len );

HHSCLNT_API int hhsAccumulateIthFitDetails( Utterance* utt, int i, int *arrayOf3Ints, int cutoff is(25) );
// accumulates the score details over the frames of the utterance.
// cutOff is subtracted from each frame score: scr -= cutoff, if( scr<0 ) scr=0;
// so cutOff should be a "good" score -> differences above good are ignored

// functions to manipulate state machines
HHSCLNT_API int hhsSetUserRulesOfState( const char *stateName, void** usrRuleArray, int lenUsrRuleArray );
#define hhsDeleteState( stateName ) ( hhsSetUserRulesOfState( stateName, 0, 0 ), 1 )
HHSCLNT_API int hhsMoveUserRulesOfStateToState( const char *srcName, const char *targName );
HHSCLNT_API int hhsDeleteAllRecogStates();
HHSCLNT_API int hhsSwitchStateUsers( User* oldUsr, User* toNewUsr );
HHSCLNT_API int hhsGetUserRulesOfState( const char *stateName, void** usrRuleArray, int lenUsrRuleArray );

// the named recognition state
HHSCLNT_API int hhsGetRecogState( char *stateName, int len );
HHSCLNT_API int hhsSetRecogState( const char *stateName );

// for re-recognition purposes
HHSCLNT_API int hhsRecognize( Utterance* utt, void** usrRuleArray, int lenUsrRuleArray );

typedef int (*InterimResultCallback)( const ClientInterimResult*, void* );
// nonzero return of InterimResultCallback() signals abort!

// inside the callback these functions are legal:
// (returns values are: 0==false and nonzero==true)
HHSCLNT_API int hhsCallingFromInitialSilence( const ClientInterimResult *ir );
HHSCLNT_API int hhsCallingFromRecognize( const ClientInterimResult *ir );
HHSCLNT_API int hhsCallingFromComplete( const ClientInterimResult *ir );

// if( hhsCallingFromRecognize() ) then these functions are legal:
HHSCLNT_API int hhsInterimNumFramesInUtt( const ClientInterimResult *ir );
HHSCLNT_API User* hhsInterimResultUser( const ClientInterimResult *ir );
HHSCLNT_API int hhsInterimResultScore( const ClientInterimResult *ir );
HHSCLNT_API const RuleData** hhsInterimResultRules( const ClientInterimResult* ir );
HHSCLNT_API const char* hhsInterimResultToAscii( const ClientInterimResult* ir );

// hhsListen() only exists for true Client/Server builds (not direct builds)
HHSCLNT_API Utterance* hhsListen( int lstnFlag, short* rtnStatus is(0), InterimResultCallback callBk is(0), void* callBkData is(0) );

// hhsListenDirect() and hhsListenDirectPolled() are direct build functions
// the direct build currently does not support InterimResultCallback's
HHSCLNT_API int hhsListenDirect( int lstnFlag, OnSpeechCallback onspeech, void* mydata );
HHSCLNT_API Utterance* hhsListenDirectPolled( int lstnFlag, short* rtnStatus );

HHSCLNT_API int hhsIsListening();	// doesnt care in silence or utt
HHSCLNT_API int hhsIsInUtt();
HHSCLNT_API int hhsStopListening();

HHSCLNT_API int hhsListenPause();
HHSCLNT_API int hhsListenResume();
HHSCLNT_API int hhsIsPaused();

HHSCLNT_API int hhsStringToWords( const char *string, WordData** words, int wordsLen );
HHSCLNT_API WordData* hhsGetNextAltWord( WordData* wrd );
HHSCLNT_API int hhsRetireWord( const char *spelling );
HHSCLNT_API int hhsGetWordSpelling( WordData* wrd, char *buf, int blen );
HHSCLNT_API int hhsGetWordPron( WordData* wrd, char *buf, int blen );
HHSCLNT_API int hhsGetRuleName( RuleData* rul, char *buf, int blen );
HHSCLNT_API int hhsIthResultElementCount( Utterance* utt, int i );
HHSCLNT_API int hhsIthResultElements( Utterance* utt, int i, unsigned *elements, int elemLen );
// elemLen should be a multiple of REsize, ResultElements are 3 or 4 unsigned's.

HHSCLNT_API WordDictionary* hhsNewWordDictionary( short langId );
HHSCLNT_API int hhsDeleteWordDictionary( WordDictionary* wd );
HHSCLNT_API WordDictionary* hhsGetWordDictionary();
HHSCLNT_API WordDictionary* hhsSelectWordDictionary( WordDictionary* nxt );
HHSCLNT_API WordData* hhsNewWord( const char *spelling, const char *pron, int penalty );
HHSCLNT_API int hhsAddToAlternative( RuleData* rl, int index, const char *bodyStr );
HHSCLNT_API int hhsLoadPronRule( RuleData* rl, const char *spelling, WordDictionary* abcDict );


#ifdef __cplusplus
}
#endif

#undef is

#endif 	//ifdef hhsClnt_h
